# TensorFlow with IBM PowerAI Distributed Deep Learning (DDL)

---

IBM PowerAI Distributed Deep Learning (DDL) custom operator for TensorFlow.
The DDL custom operator uses IBM Spectrum MPI and NCCL to provide high-speed
communications for distributed TensorFlow.

The DDL custom operator can be found in the `ddl-tensorflow` package.
For more information about DDL and about the TensorFlow operator, see:

- `$CONDA_PREFIX/doc/ddl/README.md`
- `$CONDA_PREFIX/doc/ddl-tensorflow/README-API.md`

## Model Integration

For a short tutorial on how to use DDL with Tensorflow, see:

- `$CONDA_PREFIX/doc/ddl-tensorflow/Tutorial.md`

The package includes examples of models enabled with DDL. To install the examples in a user directory, run:

```bash
$ ddl-tensorflow-install-samples <somedir>
```

For more information on Slim model integration, see:

- `$CONDA_PREFIX/doc/ddl-tensorflow/README-slim.md`

For Convolutional Network implementation example using TensorFlow library with
the MNIST database of handwritten digits, see:

- `$CONDA_PREFIX/doc/ddl-tensorflow/README-mnist.md`

For implementation example using TensorFlow-keras library with
the MNIST database of handwritten digits, see:

- `$CONDA_PREFIX/doc/ddl-tensorflow/README-keras.md`

PowerAI also provides a version of TensorFlow High Performance Models that is integrated with DDL. For information on the TensorFlow High Performance Models scripts, see:

- `$CONDA_PREFIX/tf_cnn_benchmarks/README.md`

## TensorFlow Keras Support

DDL can be used with a TensorFlow program that uses the Tensorflow.Keras API.

One item to note is that Tensorflow.Keras will create a directory to store datasets. If this directory does not exist on all of the nodes before a DDL enabled Tensorflow.Keras program is called, deadlock will occur. To create this directory on all of the nodes, run:

```bash
ddlrun -H node1,...,nodeN mkdir -p ~/.keras/datasets
```

## CPU Only Support

DDL can be used with TensorFlow on the CPU. CUDA will still need to be installed on each node, but the nodes do not need to have GPUs. In order to run on the CPU, the `-cpu_only` flag to ddlrun must be set correctly, and CUDA_VISIBLE_DEVICES must be set to empty for all of the nodes. NOTE: Performance on the CPU will be much slower than performance on a GPU. Below is an example:

```bash
ddlrun -H node1,node2 --ddlarg "-cpu_only 1" --mpiarg "-x CUDA_VISIBLE_DEVICES=" python mnist-env.py
```

## TensorFlow Distribution Strategies Support

DDL can be used with TensorFlow's Distribution Strategies. DDL only works with the CollectiveAllReduceStrategy and the number of GPU workers must be set to 1. For an example of this, please see:

- `$CONDA_PREFIX/doc/ddl-tensorflow/README-estimator.md`

NOTE: TensorFlow's Distribution Strategies currently is incompatible with tf.contrib.layers.batch_norm and the slim framework.

&copy; Copyright IBM Corporation 2018, 2019

